#version 300 es
precision highp float;
precision highp int;

struct SurfaceParams
{
    mediump vec3 albedo;
    mediump float opacity;
    mediump float cutoff;
    mediump vec3 emissive;
    mediump vec2 metalParams;
    mediump vec3 roughParams;
    mediump vec3 clearCoatRoughParams;
    mediump vec2 occParams;
    mediump vec3 diffCol;
    mediump vec3 specCol;
    mediump vec2 anisoParams;
    mediump float thin;
    mediump float subsurface;
    mediump vec3 subsurfaceColMultiply;
    mediump vec3 subsurfaceCol;
    mediump float ior;
    mediump float transmittance;
    mediump float transmittanceColorAtDistance;
    mediump float clearCoat;
    mediump vec3 pos;
    mediump vec3 nDir;
    mediump vec3 vnDir;
    mediump vec3 cnDir;
    mediump vec3 tDir;
    mediump vec3 bDir;
    mediump vec3 vDir;
    mediump vec3 rDir;
    mediump vec3 crDir;
};

struct LightParams
{
    mediump float enable;
    mediump vec3 lDir;
    mediump vec3 color;
    mediump float intensity;
    mediump vec3 attenuate;
};

struct EnvironmentParams
{
    mediump float intensity;
    mediump float rotation;
};

struct VSOutput
{
    mediump vec3 posWS;
    mediump vec3 nDirWS;
    mediump vec3 tDirWS;
    mediump vec3 bDirWS;
};

struct ShaderGraphInput
{
    mediump float envInt;
    mediump float envRot;
    mediump vec3 albedo;
    mediump float opacity;
    mediump float cutoff;
    mediump vec3 normal;
    mediump vec3 clearCoatNormal;
    mediump float metallic;
    mediump float roughness;
    mediump float ao;
    mediump float subsurface;
    mediump vec3 subsurfaceCol;
    mediump vec3 subsurfaceColMultiply;
    mediump float ior;
    mediump float transmittance;
    mediump float transmittanceColorAtDistance;
    mediump float thin;
    mediump float clearCoat;
    mediump float clearCoatRoughness;
    mediump vec3 emissive;
    mediump float anisotropic;
    mediump float anisotropicRotate;
    mediump float rampID;
    mediump float rim;
    mediump vec3 rimCol;
    mediump vec3 ambient;
    mediump vec3 matcap;
    mediump float smoothFactor;
};

struct LightGroupParams
{
    LightParams DirLights[3];
    LightParams PointLights[8];
    LightParams SpotLights[8];
    mediump float dummy;
};

struct ParamsLogC
{
    mediump float cut;
    mediump float a;
    mediump float b;
    mediump float c;
    mediump float d;
    mediump float e;
    mediump float f;
};

uniform mediump vec3 u_WorldSpaceCameraPos;
uniform mediump float u_DirLightsEnabled[3];
uniform mediump float u_DirLightNum;
uniform mediump vec4 u_DirLightsDirection[3];
uniform mediump vec4 u_DirLightsColor[3];
uniform mediump float u_DirLightsIntensity[3];
uniform mediump float u_PointLightsEnabled[8];
uniform mediump float u_PointLightNum;
uniform mediump vec4 u_PointLightsPosition[8];
uniform mediump vec4 u_PointLightsColor[8];
uniform mediump float u_PointLightsIntensity[8];
uniform mediump float u_PointLightsAttenRangeInv[8];
uniform mediump float u_SpotLightsEnabled[8];
uniform mediump float u_SpotLightNum;
uniform mediump vec4 u_SpotLightsPosition[8];
uniform mediump vec4 u_SpotLightsColor[8];
uniform mediump float u_SpotLightsIntensity[8];
uniform mediump float u_SpotLightsAttenRangeInv[8];
uniform mediump vec4 u_SpotLightsDirection[8];
uniform mediump float u_SpotLightsOuterAngleCos[8];
uniform mediump float u_SpotLightsInnerAngleCos[8];
uniform mediump sampler2D _EnvTex;
uniform mediump sampler2D u_AmazDefaultWhiteTex;
uniform mediump mat4 u_MVP;
uniform mediump mat4 u_MV;
uniform mediump mat4 u_View;
uniform mediump mat4 u_InvView;
uniform mediump mat4 u_Projection;
uniform mediump mat4 u_VP;
uniform mediump mat4 u_TransposeMV;
uniform mediump mat4 u_InvTransposeMV;
uniform mediump mat4 u_Model;
uniform mediump mat4 u_InvModel;
uniform mediump mat4 u_TransposeInvModel;
uniform mediump vec4 u_ProjectionParams;
uniform mediump vec4 u_ScreenParams;
uniform mediump vec4 u_ZBufferParams;
uniform mediump vec4 u_OrthoParams;
uniform mediump mat4 u_CameraProjection;
uniform mediump mat4 u_CameraInvProjection;
uniform mediump vec4 u_CameraFrustumLeftPlane;
uniform mediump vec4 u_CameraFrustumRightPlane;
uniform mediump vec4 u_CameraFrustumBottomPlane;
uniform mediump vec4 u_CameraFrustumTopPlane;
uniform mediump vec4 u_CameraFrustumNearPlane;
uniform mediump vec4 u_CameraFrustumFarPlane;
uniform mediump vec4 u_Time;
uniform mediump vec4 u_SinTime;
uniform mediump vec4 u_CosTime;
uniform mediump vec4 u_DeltaTime;
uniform mediump sampler2D u_FBOTexture;
uniform mediump sampler2D u_CameraRT;
uniform mediump sampler2D u_CameraDepthTexture;
uniform mediump sampler2D u_LastCameraDepthTexture;

in mediump vec3 v_posWS;
in mediump vec3 v_nDirWS;
in mediump vec3 v_tDirWS;
in mediump vec3 v_bDirWS;
layout(location = 0) out mediump vec4 o_FragColor;
in mediump vec2 v_uv0;
in mediump vec2 v_uv1;
in mediump vec4 v_color;

mediump float saturate(mediump float x)
{
    return clamp(x, 0.0, 1.0);
}

mediump float Pow2(mediump float x)
{
    return x * x;
}

mediump float SpecularAO(SurfaceParams S)
{
    mediump float ndv = max(0.0, dot(S.nDir, S.vDir));
    mediump float visibility = S.occParams.x;
    mediump float perceptualRoughness = S.roughParams.x;
    mediump float param = (pow(ndv + visibility, exp2(((-16.0) * perceptualRoughness) - 1.0)) - 1.0) + visibility;
    mediump float lagardeAO = saturate(param);
    mediump float horizon = min(1.0 + dot(S.rDir, S.nDir), 1.0);
    mediump float horizonAO = horizon * horizon;
    return lagardeAO * horizonAO;
}

SurfaceParams BuildSurfaceParams(VSOutput V, ShaderGraphInput sgi)
{
    SurfaceParams S;
    S.albedo = sgi.albedo;
    S.opacity = sgi.opacity;
    mediump float param = sgi.metallic;
    S.metalParams.x = saturate(param);
    S.metalParams.y = 0.959999978542327880859375 * (1.0 - S.metalParams.x);
    S.roughParams.x = clamp(sgi.roughness, 0.07999999821186065673828125, 1.0);
    mediump float param_1 = S.roughParams.x;
    S.roughParams.y = Pow2(param_1);
    mediump float param_2 = S.roughParams.y;
    S.roughParams.z = Pow2(param_2);
    mediump float param_3 = sgi.ao;
    S.occParams.x = saturate(param_3);
    mediump float param_4 = sgi.thin;
    S.thin = saturate(param_4);
    S.clearCoat = sgi.clearCoat;
    S.emissive = sgi.emissive;
    S.diffCol = S.albedo * S.metalParams.y;
    S.specCol = mix(vec3(0.039999999105930328369140625), S.albedo, vec3(S.metalParams.x));
    S.subsurface = sgi.subsurface;
    S.subsurfaceColMultiply = sgi.subsurfaceColMultiply;
    S.subsurfaceCol = sgi.subsurfaceCol;
    S.ior = sgi.ior;
    S.transmittance = sgi.transmittance;
    S.transmittanceColorAtDistance = sgi.transmittanceColorAtDistance;
    S.pos = V.posWS;
    S.vnDir = V.nDirWS;
    S.nDir = sgi.normal;
    S.cnDir = sgi.clearCoatNormal;
    S.tDir = V.tDirWS;
    S.bDir = V.bDirWS;
    S.vDir = normalize(u_WorldSpaceCameraPos - V.posWS);
    mediump vec3 _1382;
    if (dot(S.vDir, S.nDir) < 0.0)
    {
        _1382 = reflect(S.vDir, S.nDir);
    }
    else
    {
        _1382 = S.vDir;
    }
    S.vDir = _1382;
    S.rDir = normalize(reflect(-S.vDir, S.nDir));
    SurfaceParams param_5 = S;
    S.occParams.y = SpecularAO(param_5);
    return S;
}

EnvironmentParams BuildEnvironmentParams(mediump float envInt, mediump float envRot)
{
    EnvironmentParams E;
    E.intensity = envInt;
    E.rotation = envRot;
    return E;
}

LightParams BuildDirLightParams(SurfaceParams S, mediump int index)
{
    LightParams ML;
    ML.enable = u_DirLightsEnabled[index] * step(float(index) + 0.5, u_DirLightNum);
    ML.lDir = normalize(-u_DirLightsDirection[index].xyz);
    ML.color = u_DirLightsColor[index].xyz;
    ML.intensity = u_DirLightsIntensity[index] * ML.enable;
    ML.attenuate = vec3(1.0);
    return ML;
}

mediump float Pow4(mediump float x)
{
    mediump float xx = x * x;
    return xx * xx;
}

LightParams BuildPointLightParams(SurfaceParams S, mediump int index)
{
    mediump vec3 lVec = vec3(0.0);
    mediump float lDist = 0.0;
    LightParams PL1;
    PL1.enable = u_PointLightsEnabled[index] * step(float(index) + 0.5, u_PointLightNum);
    lVec = u_PointLightsPosition[index].xyz - S.pos;
    lDist = length(lVec);
    PL1.lDir = lVec / vec3(lDist);
    PL1.color = u_PointLightsColor[index].xyz;
    PL1.intensity = u_PointLightsIntensity[index] * PL1.enable;
    lDist *= u_PointLightsAttenRangeInv[index];
    mediump float param = lDist;
    mediump float param_1 = 1.0 - Pow4(param);
    mediump float param_2 = saturate(param_1);
    mediump float param_3 = lDist;
    mediump float attenuate = (Pow2(param_2) * (Pow2(param_3) + 1.0)) * 0.25;
    PL1.attenuate = vec3(attenuate, attenuate, attenuate);
    return PL1;
}

LightParams BuildSpotLightParams(SurfaceParams S, mediump int index)
{
    mediump vec3 lVec = vec3(0.0);
    mediump float lDist = 0.0;
    mediump vec3 spotDir = vec3(0.0);
    mediump float angleAtten = 0.0;
    LightParams SL1;
    SL1.enable = u_SpotLightsEnabled[index] * step(float(index) + 0.5, u_SpotLightNum);
    lVec = u_SpotLightsPosition[index].xyz - S.pos;
    lDist = length(lVec);
    SL1.lDir = lVec / vec3(lDist);
    SL1.color = u_SpotLightsColor[index].xyz;
    SL1.intensity = u_SpotLightsIntensity[index] * SL1.enable;
    lDist *= u_SpotLightsAttenRangeInv[index];
    mediump float param = lDist;
    mediump float param_1 = 1.0 - Pow4(param);
    mediump float param_2 = saturate(param_1);
    mediump float param_3 = lDist;
    mediump float attenuate = (Pow2(param_2) * (Pow2(param_3) + 1.0)) * 0.25;
    spotDir = normalize(-u_SpotLightsDirection[index].xyz);
    angleAtten = max(0.0, dot(SL1.lDir, spotDir));
    attenuate *= smoothstep(u_SpotLightsOuterAngleCos[index], u_SpotLightsInnerAngleCos[index], angleAtten);
    SL1.attenuate = vec3(attenuate, attenuate, attenuate);
    return SL1;
}

LightGroupParams BuildLightGroupParams(SurfaceParams S)
{
    LightGroupParams LG;
    LG.dummy = 0.0;
    SurfaceParams param = S;
    mediump int param_1 = 0;
    LG.DirLights[0] = BuildDirLightParams(param, param_1);
    SurfaceParams param_2 = S;
    mediump int param_3 = 1;
    LG.DirLights[1] = BuildDirLightParams(param_2, param_3);
    SurfaceParams param_4 = S;
    mediump int param_5 = 2;
    LG.DirLights[2] = BuildDirLightParams(param_4, param_5);
    SurfaceParams param_6 = S;
    mediump int param_7 = 0;
    LG.PointLights[0] = BuildPointLightParams(param_6, param_7);
    SurfaceParams param_8 = S;
    mediump int param_9 = 1;
    LG.PointLights[1] = BuildPointLightParams(param_8, param_9);
    SurfaceParams param_10 = S;
    mediump int param_11 = 0;
    LG.SpotLights[0] = BuildSpotLightParams(param_10, param_11);
    SurfaceParams param_12 = S;
    mediump int param_13 = 1;
    LG.SpotLights[1] = BuildSpotLightParams(param_12, param_13);
    return LG;
}

mediump float atan2(mediump float x, mediump float y)
{
    mediump float signx = (x < 0.0) ? (-1.0) : 1.0;
    return signx * acos(clamp(y / length(vec2(x, y)), -1.0, 1.0));
}

mediump vec2 GetPanoramicTexCoordsFromDir(inout mediump vec3 dir, mediump float rotation)
{
    dir = normalize(dir);
    mediump float param = dir.x;
    mediump float param_1 = -dir.z;
    mediump vec2 uv;
    uv.x = (atan2(param, param_1) - 1.57079637050628662109375) / 6.283185482025146484375;
    uv.y = acos(dir.y) / 3.1415927410125732421875;
    uv.x += rotation;
    uv.x = fract((uv.x + floor(uv.x)) + 1.0);
    return uv;
}

mediump vec3 SamplerEncodedPanoramicWithUV(mediump sampler2D panoramic, mediump vec2 uv, mediump float lod)
{
    mediump float lodMin = floor(lod);
    mediump float lodLerp = lod - lodMin;
    mediump vec2 uvLodMin = uv;
    mediump vec2 uvLodMax = uv;
    mediump vec2 size = vec2(0.0);
    if (abs(lodMin - 0.0) < 0.001000000047497451305389404296875)
    {
        uvLodMin.x = ((((uv.x * 511.0) / 512.0) + 0.0009765625) * 1.0) + 0.0;
        uvLodMin.y = ((((uv.y * 255.0) / 256.0) + 0.001953125) * 0.5) + 0.0;
        uvLodMax.x = ((((uv.x * 255.0) / 256.0) + 0.001953125) * 0.5) + 0.0;
        uvLodMax.y = ((((uv.y * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.5;
    }
    else
    {
        if (abs(lodMin - 1.0) < 0.001000000047497451305389404296875)
        {
            uvLodMin.x = ((((uv.x * 255.0) / 256.0) + 0.001953125) * 0.5) + 0.0;
            uvLodMin.y = ((((uv.y * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.5;
            uvLodMax.x = ((((uv.x * 255.0) / 256.0) + 0.001953125) * 0.5) + 0.5;
            uvLodMax.y = ((((uv.y * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.5;
        }
        else
        {
            if (abs(lodMin - 2.0) < 0.001000000047497451305389404296875)
            {
                uvLodMin.x = ((((uv.x * 255.0) / 256.0) + 0.001953125) * 0.5) + 0.5;
                uvLodMin.y = ((((uv.y * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.5;
                uvLodMax.x = ((((uv.x * 255.0) / 256.0) + 0.001953125) * 0.5) + 0.0;
                uvLodMax.y = ((((uv.y * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.75;
            }
            else
            {
                if (abs(lodMin - 3.0) < 0.001000000047497451305389404296875)
                {
                    uvLodMin.x = ((((uv.x * 255.0) / 256.0) + 0.001953125) * 0.5) + 0.0;
                    uvLodMin.y = ((((uv.y * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.75;
                    uvLodMax.x = ((((uv.x * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.5;
                    uvLodMax.y = ((((uv.y * 63.0) / 64.0) + 0.0078125) * 0.125) + 0.75;
                }
                else
                {
                    if (abs(lodMin - 4.0) < 0.001000000047497451305389404296875)
                    {
                        uvLodMin.x = ((((uv.x * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.5;
                        uvLodMin.y = ((((uv.y * 63.0) / 64.0) + 0.0078125) * 0.125) + 0.75;
                        uvLodMax.x = ((((uv.x * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.75;
                        uvLodMax.y = ((((uv.y * 63.0) / 64.0) + 0.0078125) * 0.125) + 0.75;
                    }
                    else
                    {
                        if (abs(lodMin - 5.0) < 0.001000000047497451305389404296875)
                        {
                            uvLodMin.x = ((((uv.x * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.75;
                            uvLodMin.y = ((((uv.y * 63.0) / 64.0) + 0.0078125) * 0.125) + 0.75;
                            uvLodMax.x = ((((uv.x * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.5;
                            uvLodMax.y = ((((uv.y * 63.0) / 64.0) + 0.0078125) * 0.125) + 0.875;
                        }
                        else
                        {
                            if (abs(lodMin - 6.0) < 0.001000000047497451305389404296875)
                            {
                                uvLodMin.x = ((((uv.x * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.5;
                                uvLodMin.y = ((((uv.y * 63.0) / 64.0) + 0.0078125) * 0.125) + 0.875;
                                uvLodMax.x = ((((uv.x * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.75;
                                uvLodMax.y = ((((uv.y * 63.0) / 64.0) + 0.0078125) * 0.125) + 0.875;
                            }
                            else
                            {
                                if (abs(lodMin - 7.0) < 0.001000000047497451305389404296875)
                                {
                                    uvLodMin.x = ((((uv.x * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.75;
                                    uvLodMin.y = ((((uv.y * 63.0) / 64.0) + 0.0078125) * 0.125) + 0.875;
                                    uvLodMax.x = ((((uv.x * 127.0) / 128.0) + 0.00390625) * 0.25) + 0.75;
                                    uvLodMax.y = ((((uv.y * 63.0) / 64.0) + 0.0078125) * 0.125) + 0.875;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    mediump vec4 envEncoded = mix(texture(panoramic, uvLodMin), texture(panoramic, uvLodMax), vec4(lodLerp));
    return envEncoded.xyz / vec3(envEncoded.w);
}

mediump vec3 SamplerEncodedPanoramic(mediump sampler2D panoramic, mediump vec3 dir, mediump float rotation, mediump float lod)
{
    mediump vec3 param = dir;
    mediump float param_1 = rotation;
    mediump vec2 _1170 = GetPanoramicTexCoordsFromDir(param, param_1);
    mediump vec2 uv = _1170;
    mediump vec2 param_2 = uv;
    mediump float param_3 = lod;
    return SamplerEncodedPanoramicWithUV(panoramic, param_2, param_3);
}

mediump vec3 GTAO_MultiBounce(mediump float visibility, mediump vec3 albedo)
{
    mediump vec3 a = (albedo * 2.040400028228759765625) - vec3(0.3323999941349029541015625);
    mediump vec3 b = (albedo * (-4.79510021209716796875)) + vec3(0.6417000293731689453125);
    mediump vec3 c = (albedo * 2.755199909210205078125) + vec3(0.69029998779296875);
    return max(vec3(visibility), ((((a * visibility) + b) * visibility) + c) * visibility);
}

mediump vec3 Diffuse_Panoramic(SurfaceParams S, EnvironmentParams E, mediump sampler2D envTex)
{
    mediump vec3 param = S.nDir;
    mediump float param_1 = E.rotation;
    mediump float param_2 = 7.0;
    mediump vec3 lighting = SamplerEncodedPanoramic(envTex, param, param_1, param_2);
    mediump float param_3 = S.occParams.x;
    mediump vec3 param_4 = S.diffCol;
    mediump vec3 multiBounceColor = GTAO_MultiBounce(param_3, param_4);
    return ((lighting * S.diffCol) * multiBounceColor) * E.intensity;
}

mediump vec3 EnvBRDFApprox(SurfaceParams S)
{
    mediump float ndv = max(0.0, dot(S.nDir, S.vDir));
    mediump float perceptualRoughness = S.roughParams.x;
    mediump vec4 r = (vec4(-1.0, -0.0274999998509883880615234375, -0.572000026702880859375, 0.02199999988079071044921875) * perceptualRoughness) + vec4(1.0, 0.0425000004470348358154296875, 1.03999996185302734375, -0.039999999105930328369140625);
    mediump float a004 = (min(r.x * r.x, exp2((-9.27999973297119140625) * ndv)) * r.x) + r.y;
    mediump vec2 AB = (vec2(-1.03999996185302734375, 1.03999996185302734375) * a004) + r.zw;
    mediump float param = 50.0 * S.specCol.y;
    AB.y *= saturate(param);
    return (S.specCol * AB.x) + vec3(AB.y);
}

mediump vec3 Specular_Panoramic(SurfaceParams S, EnvironmentParams E, mediump sampler2D envTex)
{
    mediump vec3 dir = mix(S.rDir, S.nDir, vec3(S.roughParams.x * S.roughParams.y));
    mediump vec3 param = dir;
    mediump float param_1 = E.rotation;
    mediump float param_2 = (S.roughParams.x - 0.07999999821186065673828125) * 7.0;
    mediump vec3 specEnv = SamplerEncodedPanoramic(envTex, param, param_1, param_2);
    SurfaceParams param_3 = S;
    mediump vec3 brdf = EnvBRDFApprox(param_3);
    mediump float param_4 = S.occParams.y;
    mediump vec3 param_5 = S.specCol;
    mediump vec3 multiBounceColor = GTAO_MultiBounce(param_4, param_5);
    return ((brdf * specEnv) * multiBounceColor) * E.intensity;
}

mediump vec3 Diffuse_OrenNayar(SurfaceParams S, LightParams L)
{
    mediump vec3 hDir = normalize(L.lDir + S.vDir);
    mediump float ndl = max(0.0, dot(S.nDir, L.lDir));
    mediump float ndv = max(0.0, dot(S.nDir, S.vDir));
    mediump float vdh = max(0.0, dot(S.vDir, hDir));
    mediump float a = S.roughParams.y;
    mediump float s = a;
    mediump float s2 = s * s;
    mediump float VoL = ((2.0 * vdh) * vdh) - 1.0;
    mediump float Cosri = VoL - (ndv * ndl);
    mediump float C1 = 1.0 - ((0.5 * s2) / (s2 + 0.3300000131130218505859375));
    mediump float _361;
    if (Cosri >= 0.0)
    {
        _361 = 1.0 / max(ndl, ndv);
    }
    else
    {
        _361 = 1.0;
    }
    mediump float C2 = (((0.449999988079071044921875 * s2) / (s2 + 0.0900000035762786865234375)) * Cosri) * _361;
    mediump float lighting = ((C1 + C2) * (1.0 + (S.roughParams.x * 0.5))) * ndl;
    return (((S.diffCol * L.color) * L.intensity) * L.attenuate) * lighting;
}

mediump float V_SmithJointApprox(mediump float a, mediump float ndv, mediump float ndl)
{
    mediump float lambdaV = ndl * ((ndv * (1.0 - a)) + a);
    mediump float lambdaL = ndv * ((ndl * (1.0 - a)) + a);
    return 0.5 / ((lambdaV + lambdaL) + 9.9999997473787516355514526367188e-06);
}

mediump float D_GGX(mediump float ndh, mediump float a2)
{
    mediump float d = (((ndh * a2) - ndh) * ndh) + 1.0;
    return (a2 * 0.31830990314483642578125) / ((d * d) + 1.0000000116860974230803549289703e-07);
}

mediump float Pow5(mediump float x)
{
    mediump float xx = x * x;
    return (xx * xx) * x;
}

mediump vec3 F_Schlick(mediump vec3 f0, mediump float vdh)
{
    mediump float param = 1.0 - vdh;
    mediump float t = Pow5(param);
    return f0 + ((vec3(1.0) - f0) * t);
}

mediump vec3 Specular_GGX(SurfaceParams S, LightParams L)
{
    mediump vec3 hDir = normalize(L.lDir + S.vDir);
    mediump float ndh = max(0.0, dot(S.nDir, hDir));
    mediump float vdh = max(0.0, dot(S.vDir, hDir));
    mediump float ndl = max(0.0, dot(S.nDir, L.lDir));
    mediump float ndv = max(0.0, dot(S.nDir, S.vDir));
    mediump float a = S.roughParams.y;
    mediump float a2 = S.roughParams.z;
    mediump float param = ndl;
    mediump float param_1 = ndv;
    mediump float param_2 = a;
    mediump float V = V_SmithJointApprox(param, param_1, param_2);
    mediump float param_3 = ndh;
    mediump float param_4 = a2;
    mediump float D = D_GGX(param_3, param_4);
    mediump vec3 param_5 = S.specCol;
    mediump float param_6 = vdh;
    mediump vec3 F = F_Schlick(param_5, param_6);
    mediump vec3 specular = ((((((F * V) * D) * 3.1415927410125732421875) * ndl) * L.color) * L.intensity) * L.attenuate;
    return specular;
}

mediump vec3 Diffuse_Lambert(SurfaceParams S, LightParams L)
{
    mediump float ndl = max(0.0, dot(S.nDir, L.lDir));
    mediump float lighting = ndl;
    return (((S.diffCol * L.color) * L.intensity) * L.attenuate) * lighting;
}

mediump float V_Const()
{
    return 0.25;
}

mediump vec3 Specular_GGX_Low(SurfaceParams S, LightParams L)
{
    mediump vec3 hDir = normalize(L.lDir + S.vDir);
    mediump float ndh = max(0.0, dot(S.nDir, hDir));
    mediump float vdh = max(0.0, dot(S.vDir, hDir));
    mediump float ndl = max(0.0, dot(S.nDir, L.lDir));
    mediump float a = S.roughParams.y;
    mediump float a2 = S.roughParams.z;
    mediump float V = V_Const();
    mediump float param = ndh;
    mediump float param_1 = a2;
    mediump float D = D_GGX(param, param_1);
    mediump vec3 param_2 = S.specCol;
    mediump float param_3 = vdh;
    mediump vec3 F = F_Schlick(param_2, param_3);
    mediump vec3 specular = ((((((F * V) * D) * 3.1415927410125732421875) * ndl) * L.color) * L.intensity) * L.attenuate;
    return specular;
}

mediump vec3 Lighting(VSOutput V, SurfaceParams S, EnvironmentParams E, mediump sampler2D envTex, LightGroupParams LG)
{
    mediump vec3 Fd = vec3(0.0);
    mediump vec3 Fr = vec3(0.0);
    mediump float coatAttenuate_IBL = 1.0;
    SurfaceParams param = S;
    EnvironmentParams param_1 = E;
    Fd += (Diffuse_Panoramic(param, param_1, envTex) * coatAttenuate_IBL);
    SurfaceParams param_2 = S;
    EnvironmentParams param_3 = E;
    Fr += (Specular_Panoramic(param_2, param_3, envTex) * coatAttenuate_IBL);
    LightParams ML = LG.DirLights[0];
    if (ML.enable > 0.5)
    {
        mediump float coatAttenuate_ML = 1.0;
        SurfaceParams param_4 = S;
        LightParams param_5 = ML;
        Fd += (Diffuse_OrenNayar(param_4, param_5) * coatAttenuate_ML);
        SurfaceParams param_6 = S;
        LightParams param_7 = ML;
        Fr += (Specular_GGX(param_6, param_7) * coatAttenuate_ML);
    }
    for (mediump int i = 1; i < 3; i++)
    {
        LightParams DL = LG.DirLights[i];
        if (DL.enable > 0.5)
        {
            mediump float coatAttenuate_DL = 1.0;
            SurfaceParams param_8 = S;
            LightParams param_9 = DL;
            Fd += (Diffuse_Lambert(param_8, param_9) * coatAttenuate_DL);
            SurfaceParams param_10 = S;
            LightParams param_11 = DL;
            Fr += (Specular_GGX_Low(param_10, param_11) * coatAttenuate_DL);
        }
    }
    for (mediump int i_1 = 0; i_1 < 8; i_1++)
    {
        LightParams PL = LG.PointLights[i_1];
        if (PL.enable > 0.5)
        {
            mediump float coatAttenuate_PL = 1.0;
            SurfaceParams param_12 = S;
            LightParams param_13 = PL;
            Fd += (Diffuse_Lambert(param_12, param_13) * coatAttenuate_PL);
            SurfaceParams param_14 = S;
            LightParams param_15 = PL;
            Fr += (Specular_GGX_Low(param_14, param_15) * coatAttenuate_PL);
        }
    }
    for (mediump int i_2 = 0; i_2 < 8; i_2++)
    {
        LightParams SL = LG.SpotLights[i_2];
        if (SL.enable > 0.5)
        {
            mediump float coatAttenuate_SL = 1.0;
            SurfaceParams param_16 = S;
            LightParams param_17 = SL;
            Fd += (Diffuse_Lambert(param_16, param_17) * coatAttenuate_SL);
            SurfaceParams param_18 = S;
            LightParams param_19 = SL;
            Fr += (Specular_GGX_Low(param_18, param_19) * coatAttenuate_SL);
        }
    }
    mediump vec3 finalRGB = Fd + Fr;
    return finalRGB;
}

mediump vec4 MainEntry(mediump sampler2D envTex, VSOutput V, ShaderGraphInput sgi)
{
    VSOutput param = V;
    ShaderGraphInput param_1 = sgi;
    SurfaceParams S = BuildSurfaceParams(param, param_1);
    mediump float param_2 = sgi.envInt;
    mediump float param_3 = sgi.envRot;
    EnvironmentParams E = BuildEnvironmentParams(param_2, param_3);
    SurfaceParams param_4 = S;
    LightGroupParams LG = BuildLightGroupParams(param_4);
    VSOutput param_5 = V;
    SurfaceParams param_6 = S;
    EnvironmentParams param_7 = E;
    LightGroupParams param_8 = LG;
    mediump vec3 finalRGB = Lighting(param_5, param_6, param_7, envTex, param_8);
    mediump vec4 result = vec4(finalRGB, S.opacity);
    return result;
}

mediump vec3 SafePow(inout mediump vec3 v, inout mediump vec3 e)
{
    v = max(v, vec3(9.9999997473787516355514526367188e-06));
    e = max(e, vec3(9.9999997473787516355514526367188e-06));
    return pow(v, e);
}

mediump vec3 LinearToGamma(mediump vec3 col)
{
    mediump vec3 param = col;
    mediump vec3 param_1 = vec3(0.4545454680919647216796875);
    mediump vec3 _250 = SafePow(param, param_1);
    return _250;
}

mediump vec4 ApplyBlendMode(mediump vec4 color)
{
    mediump vec4 ret = color;
    return ret;
}

void main()
{
    mediump vec3 portvar_65c9f34a = vec3(0.666117012500762939453125, 0.4368127882480621337890625, 0.0342302061617374420166015625);
    mediump vec4 portvar_51fd9f08 = vec4(0.0);
    VSOutput temp_temp_1;
    temp_temp_1.posWS = v_posWS;
    temp_temp_1.nDirWS = normalize(v_nDirWS);
    temp_temp_1.tDirWS = normalize(v_tDirWS);
    temp_temp_1.bDirWS = normalize(v_bDirWS);
    ShaderGraphInput temp_temp_2;
    temp_temp_2.albedo = portvar_65c9f34a;
    temp_temp_2.metallic = 0.800000011920928955078125;
    temp_temp_2.roughness = 0.5;
    temp_temp_2.normal = mat3(vec3(temp_temp_1.tDirWS), vec3(temp_temp_1.bDirWS), vec3(temp_temp_1.nDirWS)) * vec3(0.0, 0.0, 1.0);
    temp_temp_2.ao = 1.0;
    temp_temp_2.opacity = 1.0;
    temp_temp_2.cutoff = 0.0;
    temp_temp_2.envInt = 1.0;
    temp_temp_2.envRot = 0.0;
    temp_temp_2.emissive = vec3(0.0);
    VSOutput param = temp_temp_1;
    ShaderGraphInput param_1 = temp_temp_2;
    portvar_51fd9f08 = MainEntry(_EnvTex, param, param_1);
    mediump vec4 temp_temp_3 = portvar_51fd9f08;
    mediump vec3 param_2 = temp_temp_3.xyz;
    mediump vec3 _1959 = LinearToGamma(param_2);
    temp_temp_3 = vec4(_1959.x, _1959.y, _1959.z, temp_temp_3.w);
    mediump vec4 param_3 = temp_temp_3;
    temp_temp_3 = ApplyBlendMode(param_3);
    o_FragColor = temp_temp_3;
}

