#pragma clang diagnostic ignored "-Wmissing-prototypes"
#pragma clang diagnostic ignored "-Wmissing-braces"

#include <metal_stdlib>
#include <simd/simd.h>

using namespace metal;

template<typename T, size_t Num>
struct spvUnsafeArray
{
    T elements[Num ? Num : 1];
    
    thread T& operator [] (size_t pos) thread
    {
        return elements[pos];
    }
    constexpr const thread T& operator [] (size_t pos) const thread
    {
        return elements[pos];
    }
    
    device T& operator [] (size_t pos) device
    {
        return elements[pos];
    }
    constexpr const device T& operator [] (size_t pos) const device
    {
        return elements[pos];
    }
    
    constexpr const constant T& operator [] (size_t pos) const constant
    {
        return elements[pos];
    }
    
    threadgroup T& operator [] (size_t pos) threadgroup
    {
        return elements[pos];
    }
    constexpr const threadgroup T& operator [] (size_t pos) const threadgroup
    {
        return elements[pos];
    }
};

struct buffer_t
{
    float4x4 u_TransposeInvModel;
    float4x4 u_Model;
    spvUnsafeArray<float4x4, 50> u_Palatte;
    float4x4 u_VP;
};

struct main0_out
{
    float3 v_posWS;
    float3 v_nDirWS;
    float3 v_tDirWS;
    float3 v_bDirWS;
    float2 v_uv0;
    float2 v_uv1;
    float4 v_color;
    float4 gl_Position [[position]];
};

struct main0_in
{
    float3 attPosition [[attribute(0)]];
    float3 attNormal [[attribute(1)]];
    float3 attTangent [[attribute(2)]];
    float2 attTexcoord0 [[attribute(3)]];
    float2 attTexcoord1 [[attribute(4)]];
    float4 attColor [[attribute(5)]];
    float4 attBoneIds [[attribute(6)]];
    float4 attWeights [[attribute(7)]];
};

static inline __attribute__((always_inline))
float4 objectToWorldNormal(thread const float4& objectNormal, constant float4x4& u_TransposeInvModel)
{
    return float4(normalize((u_TransposeInvModel * objectNormal).xyz), objectNormal.w);
}

static inline __attribute__((always_inline))
float3 objectToWorldNormal(thread const float3& objectNormal, constant float4x4& u_TransposeInvModel)
{
    float4 param = float4(objectNormal, 0.0);
    return objectToWorldNormal(param, u_TransposeInvModel).xyz;
}

static inline __attribute__((always_inline))
float3 objectToWorldDir(thread const float3& objectDir, constant float4x4& u_Model)
{
    return normalize(float3x3(u_Model[0].xyz, u_Model[1].xyz, u_Model[2].xyz) * objectDir);
}

static inline __attribute__((always_inline))
float3 objectToWorldTangent(thread const float3& objectTangent, constant float4x4& u_Model)
{
    float3 param = objectTangent;
    return objectToWorldDir(param, u_Model);
}

vertex main0_out main0(main0_in in [[stage_in]], constant buffer_t& buffer)
{
    main0_out out = {};
    float4x4 _94 = buffer.u_Palatte[int(in.attBoneIds.x)] * in.attWeights.x;
    float4x4 _103 = buffer.u_Palatte[int(in.attBoneIds.y)] * in.attWeights.y;
    float4x4 _116 = float4x4(_94[0] + _103[0], _94[1] + _103[1], _94[2] + _103[2], _94[3] + _103[3]);
    float4x4 _125 = buffer.u_Palatte[int(in.attBoneIds.z)] * in.attWeights.z;
    float4x4 _138 = float4x4(_116[0] + _125[0], _116[1] + _125[1], _116[2] + _125[2], _116[3] + _125[3]);
    float4x4 _146 = buffer.u_Palatte[int(in.attBoneIds.w)] * in.attWeights.w;
    float4x4 boneTransform = float4x4(_138[0] + _146[0], _138[1] + _146[1], _138[2] + _146[2], _138[3] + _146[3]);
    float3 bm_postiton = (boneTransform * float4(in.attPosition, 1.0)).xyz;
    float3 bn_normal = (boneTransform * float4(in.attNormal, 0.0)).xyz;
    out.v_posWS = (buffer.u_Model * float4(bm_postiton, 1.0)).xyz;
    float3 param = bn_normal;
    out.v_nDirWS = objectToWorldNormal(param, buffer.u_TransposeInvModel);
    float3 bm_tangent = (boneTransform * float4(in.attTangent, 0.0)).xyz;
    float3 param_1 = bm_tangent;
    out.v_tDirWS = objectToWorldTangent(param_1, buffer.u_Model);
    out.v_bDirWS = cross(out.v_nDirWS, out.v_tDirWS);
    out.v_uv0 = in.attTexcoord0;
    out.v_uv1 = in.attTexcoord1;
    out.v_color = in.attColor;
    out.v_bDirWS = cross(out.v_nDirWS, out.v_tDirWS);
    float4 temp_temp_2 = buffer.u_VP * float4(out.v_posWS, 1.0);
    out.gl_Position = temp_temp_2;
    out.gl_Position.z = (out.gl_Position.z + out.gl_Position.w) * 0.5;       // Adjust clip-space for Metal
    return out;
}

