#pragma clang diagnostic ignored "-Wmissing-prototypes"

#include <metal_stdlib>
#include <simd/simd.h>

using namespace metal;

struct buffer_t
{
    float4x4 u_TransposeInvModel;
    float4x4 u_Model;
    float4x4 u_MVP;
    float4x4 u_VP;
};

struct main0_out
{
    float3 v_posWS;
    float3 v_nDirWS;
    float3 v_tDirWS;
    float3 v_bDirWS;
    float2 v_uv0;
    float2 v_uv1;
    float4 v_color;
    float4 gl_Position [[position]];
};

struct main0_in
{
    float3 attPosition [[attribute(0)]];
    float3 attNormal [[attribute(1)]];
    float3 attTangent [[attribute(2)]];
    float2 attTexcoord0 [[attribute(3)]];
    float2 attTexcoord1 [[attribute(4)]];
    float4 attColor [[attribute(5)]];
};

static inline __attribute__((always_inline))
float4 objectToWorldNormal(thread const float4& objectNormal, constant float4x4& u_TransposeInvModel)
{
    return float4(normalize((u_TransposeInvModel * objectNormal).xyz), objectNormal.w);
}

static inline __attribute__((always_inline))
float3 objectToWorldNormal(thread const float3& objectNormal, constant float4x4& u_TransposeInvModel)
{
    float4 param = float4(objectNormal, 0.0);
    return objectToWorldNormal(param, u_TransposeInvModel).xyz;
}

static inline __attribute__((always_inline))
float3 objectToWorldDir(thread const float3& objectDir, constant float4x4& u_Model)
{
    return normalize(float3x3(u_Model[0].xyz, u_Model[1].xyz, u_Model[2].xyz) * objectDir);
}

static inline __attribute__((always_inline))
float3 objectToWorldTangent(thread const float3& objectTangent, constant float4x4& u_Model)
{
    float3 param = objectTangent;
    return objectToWorldDir(param, u_Model);
}

vertex main0_out main0(main0_in in [[stage_in]], constant buffer_t& buffer)
{
    main0_out out = {};
    out.v_uv0 = in.attTexcoord0;
    out.v_uv1 = in.attTexcoord1;
    out.v_posWS = (buffer.u_Model * float4(in.attPosition, 1.0)).xyz;
    float4x4 temp_temp_1 = buffer.u_MVP;
    float3 param = in.attNormal;
    out.v_nDirWS = objectToWorldNormal(param, buffer.u_TransposeInvModel);
    float3 param_1 = in.attTangent;
    out.v_tDirWS = objectToWorldTangent(param_1, buffer.u_Model);
    out.v_bDirWS = cross(out.v_nDirWS, out.v_tDirWS);
    out.v_color = in.attColor;
    out.v_bDirWS = cross(out.v_nDirWS, out.v_tDirWS);
    float4 temp_temp_2 = buffer.u_VP * float4(out.v_posWS, 1.0);
    out.gl_Position = temp_temp_2;
    out.gl_Position.z = (out.gl_Position.z + out.gl_Position.w) * 0.5;       // Adjust clip-space for Metal
    return out;
}

